// CSE165 Kinect Starter Code
// @author Matteo Mannino, mtmannin@eng.ucsd.edu

#include <Windows.h>
#include <NuiApi.h>

#include <osgViewer/Viewer>
#include <osg/ShapeDrawable>

#include <osgViewer/ViewerEventHandlers>
//#include <osgGA/StateSetManipulator>
#include <osg/Node>
#include <osg/Group>
#include <osg/PositionAttitudeTransform>
#include <osgDB/ReadFile>
#include <osgGA/TrackballManipulator>

using namespace std;

int main()
{
	//Creating the viewer	
    osgViewer::Viewer viewer;
	
	// Window size. Comment this out if you want full screen.
	viewer.setUpViewInWindow(100,100,800,800);
	
	//Creating the root node
	osg::ref_ptr<osg::Group> root (new osg::Group);
	
	// load campus map model
	osg::Node* campusNode = osgDB::readNodeFile("./fallenstar/fallenstar.wrl");
	if(campusNode == NULL) {
		std::cout << "Failed to find campus file" << std::endl;
		system("pause");
		return -1;
	}
	
	// load geometric box primitive
	Geode* gblock = new Geode();
	gblock->addDrawable(new ShapeDrawable(new Box()));
	
	// create and set transforms
	// PositionAttitudeTransform is a type of MatrixTransform that makes it easy
	// to set the rotation (attitude) and translation (position) of the node
	PositionAttitudeTransform* campusTrans = new PositionAttitudeTransform();
	campusTrans->setAttitude(Quat(3.14159/2, Vec3d(1,0,0))); // rotate map to x-y plane
	PositionAttitudeTransform* blockTrans = new PositionAttitudeTransform();
	blockTrans->setScale(Vec3d(50,50,100));
	// right next to the CSE Building entrance, you will have to manipulate the trackball to get there
	blockTrans->setPosition(Vec3d(37000,35000,30)); 
	// Note that when you use the mouse to move the scene, the block moves with it. That
	// is because the TrackballManipulator transforms the root node, which is a parent to both
	// the campus transform and the block transform.

	// ****
	// now we build the scene graph
	// ****
	root->addChild(campusTrans);
	root->addChild(blockTrans);
	campusTrans->addChild(campusNode);
	blockTrans->addChild(gblock);
	
	// ****
	// set up call backs
	// ****
	// OSG is a little confusing when it comes to callbacks
	// This is an example for you to get a basic understanding, where we will move
	// the block around with the keyboard.

	// Basically, NodeInputState is a class we defined to describe the current user input for a Node
	NodeInputState* nodeInputState = new NodeInputState;
	// The keyboard will update this, so we defined a KeyboardEventHandler, that inherets from an 
	// osg::GUIEventHandler, to take a node input state. When the keyboard is pressed, this event will
	// trigger and change nodeInputState.
	KeyboardEventHandler* keyboardeventhandler = new KeyboardEventHandler(nodeInputState);
	viewer.addEventHandler(keyboardeventhandler);
	// Whatever changes happen to nodeInputState, we want to make to the block transform node to
	// move the block. The Update Callback for each node is called once per frame. We
	// defined a NodeEventCallback, that inherets from osg::NodeCallback, to take in a
	// NodeInputState to use for transforming itself. Remember, nodeInputState is 
	// modified by the keyboard.
	blockTrans->setUpdateCallback(new NodeEventCallback(nodeInputState));

	// set root node to viewer
	viewer.setSceneData( root.get() ); 

	// Windows size handler
	viewer.addEventHandler(new osgViewer::WindowSizeHandler);

	// set camera manipulation to a trackball
	viewer.setCameraManipulator(new osgGA::TrackballManipulator());
	
	// init the Kinect
	// invaluable resouce: http://msdn.microsoft.com/en-us/library/hh855366.aspx
	NuiInitialize(NUI_INITIALIZE_FLAG_USES_SKELETON);
	NUI_SKELETON_FRAME playerFrame;
	
	// The main loop
	viewer.realize();
	while(!viewer.done()) {
	
		NuiSkeletonGetNextFrame(0, &playerFrame);
		
	    // ****
		// do logic and animation below
		// ****
		
		// loops through all 6 kinect skeletons
		// you can specify just one, but might be buggy - sometimes the kinect doesn't use
		// the first skeleton (index 0) even though only one is tracked
		for (int i = 0; i < 6; i++)
		{
			//std::cout << ourframe.SkeletonData[i].eTrackingState << std::endl; 
			if (playerFrame.SkeletonData[i].eTrackingState == NUI_SKELETON_TRACKED)
			{
				// stand in full view of the kinect until you see this message
				std::cout << "Skeleton tracked!" << std::endl;

				// example of getting knee position
				// other positions: http://msdn.microsoft.com/en-us/library/nuiskeleton.nui_skeleton_position_index.aspx
				std::cout << "Knee position x: " << playerFrame.SkeletonData[i].SkeletonPositions[NUI_SKELETON_POSITION_KNEE_LEFT].x << std::endl;

			}
		}

		// Moving the block example WITHOUT NodeEventCallback:
		// I just left this here to note you don't need NodeEventCallback to move nodes.
		// You can do it here, too.
		//std::cout << nodeInputState->moveRequest << std::endl;
		//if(nodeInputState->moveRequest & MOVE_UP) {
		//	Vec3d pos = blockTrans->getPosition();
		//	pos.set(pos.x(), pos.y(), pos.z() + 10);
		//	blockTrans->setPosition(pos);
		//}

		// render frame
		viewer.frame(); 
	}
	
	// shut down the Kinect
	NuiShutdown();
	
	return 0;
}